@@ -1,4 +1,4 @@
-# Copyright (c) 2008 Simplistix Ltd
+# Copyright (c) 2008-2009 Simplistix Ltd
 #
 # This Software is released under the MIT License:
 # http://www.opensource.org/licenses/mit-license.html
@@ -229,6 +229,8 @@ class BaseWriter:
         self.wtname = wtbook_name
         self.style_list = []
         self.wtsheet_names = set()
+        if not rdbook.formatting_info:
+            return
         for rdxf in rdbook.xf_list:
             wtxf = xlwt.Style.XFStyle()
             #
@@ -319,7 +321,7 @@ class BaseWriter:
         
         self.rdsheet = rdsheet
         self.wtsheet_name=wtsheet_name
-        self.wtsheet = wtsheet = self.wtbook.add_sheet(wtsheet_name)
+        self.wtsheet = wtsheet = self.wtbook.add_sheet(wtsheet_name,cell_overwrite_ok=True)
         self.wtcols = set() # keep track of which columns have had their attributes set up
         #
         # MERGEDCELLS
@@ -504,6 +506,16 @@ class StreamWriter(BaseWriter):
         self.fired = True
         return self.stream
 
+class XLWTWriter(BaseWriter):
+
+    def __init__(self):
+        self.output = []
+
+    def close(self):
+        if self.wtbook is not None:
+            self.output.append((self.wtname,self.wtbook))
+            del self.wtbook
+
 class MethodFilterMethod:
 
     def __init__(self,mf,name):
