@@ -20,13 +20,13 @@ class DummyBook:
     logfile = sys.stdout
     pickleable = False
     verbosity = 0
-    formatting_info = 0
     _xf_index_to_xl_type_map = {}
     _sheet_visibility = []
     xf_list = []
     datemode = 0
 
-    def __init__(self):
+    def __init__(self,formatting_info=0):
+        self.formatting_info=formatting_info
         self.__sheets = []
         
     def add(self,sheet):
@@ -58,5 +58,5 @@ def make_sheet(rows,book=None,name='test sheet',number=0):
                 cell_type,value = value
             else:
                 cell_type=XL_CELL_TEXT
-            sheet.put_cell(rowx,colx,cell_type,value,None)
+            sheet.put_cell(rowx,colx,cell_type,value,0)
     return sheet
